# -*- coding: utf-8 -*-
"""
Created on Mon Oct  4 12:42:57 2021

@author: Claudiu Stan

Version tracking: curated version of LG15SaveByTypeRawV2.py

This code extracts and saves, to .h5 files separate for each freezing stage,
all packed CSPAD *raw* data frames with their the event numbers and pulse
energies.

INPUTS:
    - run number, entered from the command line
    - the corresponding event timing info file must be available (EvTimeR***.mat) 
    - the corresponding freezing stage info file must be available (FzTypeR***.mat)
    
OUTPUTS:
    - the outputs are in multiple .h5 files, one saved per each stage (R***Stage*.h5)
    - event numbers and stage numbers are reported live during processing, as
      well as when the data starts to be saved
    
NOTE 1: All CSPAD data is in the raw format (without any corrections)

Nore 2: The script uses Python 3 and is specific to the LG15 experiment. The
version of Python used in the data analysis environment at SLAC, as well as 
some of the SLAC data analysis packages and data formats could be updated,
which would require modifications to keep the script functional. This script 
was last used (sucessfully) in April 2023.

"""
from psana import * # Load psana libraries
import numpy as np # Load numerics package
from scipy import io # Load matlab import/export libraries
import h5py # Load hdf5 access libraries

# Get the run number from command line
Run = input("Run number?")

# Import auxiliary run data
EvDict = {}
callN = "EvTimeR"+str(Run).zfill(3)
io.loadmat(callN,mdict=EvDict)
EvNumbers = EvDict['EvTimeNumbers']

EvDict = {}
callN = "FzTypeR"+str(Run).zfill(3)
io.loadmat(callN,mdict=EvDict)
EvSel = EvDict['EvNum']
Types = EvDict['Type']

# Mount the XTC run data, define the CSPAD data object
callN = "exp=cxig1515:run="+str(Run)+':idx' # :idx is needed, not sure why...
ds  = DataSource(callN)
run = next(ds.runs()) # Defines the run object
cspadDet = Detector('Ds1CsPad',ds.env())
gasDet = Detector('FEEGasDetEnergy') # Gas detector pulse energy

# Get the available first detector image to set the output variable
sec  = EvNumbers[np.int(EvSel[0,0]-1),1]
nsec = EvNumbers[np.int(EvSel[0,0]-1),2]
fid = EvNumbers[np.int(EvSel[0,0]-1),3]
et = EventTime(int((sec<<32)|nsec),fid)
evt = run.event(et)
det0 = cspadDet.raw(evt)  #raw unassembled image


# Find the number of events for each type, generate the output arrays, auxiliary variables
for iT in range(9):
    if np.where(Types == (iT+1))[0].size > 0: # If there are events of that type
        exec('Num' + str(iT+1) + ' = np.where(Types == (iT+1))[0].size')
        exec('AllDetType' + str(iT+1) + ' = np.zeros((det0.shape[0],det0.shape[1],det0.shape[2],Num' + str(iT+1) + '), dtype=np.int16)')
        exec('EvSel' + str(iT+1) + ' = np.zeros((Num' + str(iT+1) + '), dtype=np.uint32)')
        exec('Count' + str(iT+1) + ' = np.zeros(1, dtype=np.uint32)')
        exec('PuEnrg' + str(iT+1) + ' = np.zeros((Num' + str(iT+1) + '), dtype=np.float32)')


# Main loop for assignments
print( "Processing events in Run", str(Run).zfill(3), ":" )
for iE in range(len(EvSel)):
    if Types[iE,0] > 0:
        # Iterate counter for respective type
        exec('Count' + str(Types[iE,0]) + ' = Count' + str(Types[iE,0]) + ' + 1')
        
        # Get the X-ray detector data
        sec  = EvNumbers[np.int(EvSel[iE,0]-1),1]
        nsec = EvNumbers[np.int(EvSel[iE,0]-1),2]
        fid = EvNumbers[np.int(EvSel[iE,0]-1),3]
        et = EventTime(int((sec<<32)|nsec),fid)
        evt = run.event(et)
        pkdata = cspadDet.raw(evt)  #raw unassembled image
        # Assign the X-ray detector data to its proper variable
        exec('AllDetType' + str(Types[iE,0]) + '[:,:,:,(Count' + str(Types[iE,0]) + '[0] - 1)] = pkdata')

        # Get the pulse energy from the gas detector
        gas = gasDet.get(evt)
        if gas is None:
            Tgas = np.nan
        else:
            Tgas = 0.25*(gas.f_11_ENRC()+gas.f_12_ENRC()+gas.f_21_ENRC()+gas.f_22_ENRC())        
        # Assign the pulse energy to its proper variable
        exec('PuEnrg' + str(Types[iE,0]) + '[Count' + str(Types[iE,0]) + '[0] - 1] = Tgas')
        
        # Assign the event number to its respective list
        exec('EvSel' + str(Types[iE,0]) + '[Count' + str(Types[iE,0]) + '[0] - 1] = EvNumbers[np.int(EvSel[iE,0]-1),0]')
        print( str(EvNumbers[np.int(EvSel[iE,0]-1),0]).zfill(5), "Stage", str(Types[iE,0]) )    
        

# Data saving block
print( "Saving data..." )
for iT in range(9):
    if np.where(Types == (iT+1))[0].size > 0: # If there are events of that type

        h5f = h5py.File("R"+str(Run).zfill(3)+"Stage"+str(iT+1)+".h5", 'w')
        exec("h5f.create_dataset('DetAllType', data = AllDetType"+str(iT+1)+")")
        exec("h5f.create_dataset('EvtAllType', data = EvSel"+str(iT+1)+")")
        exec("h5f.create_dataset('EngAllType', data = PuEnrg"+str(iT+1)+")")
        h5f.close()
        print( "Type", str(iT+1), "data saved" )